import torch
import os
import subprocess
import time
import argparse
import numpy as np

import time
import inspect

def log(*arguments):
    previous_frame = inspect.currentframe().f_back
    (
        filename,
        line_number,
        function_name,
        lines,
        index,
    ) = inspect.getframeinfo(previous_frame)
    output = '%s'%time.strftime("%Y/%m/%d %H:%M:%S ", time.localtime())
    output += '{}:{}]'.format(filename, line_number)
    output = output.ljust(50)
    for aug in arguments:
        output += " " + str(aug)
    print(output)

def custom_normalize(sequence, new_min, new_max):
    min_val = np.min(sequence)
    max_val = np.max(sequence)
    
    normalized_sequence = new_min + ((sequence - min_val) * (new_max - new_min)) / (max_val - min_val)
    
    return normalized_sequence

def sine_curve():
    x1 = np.linspace(0, 1, 20)
    y1 = 0.8 * x1**2
    y1 = y1 + np.random.normal(0, 0.03, len(x1))
    y1 = custom_normalize(y1, 0, 0.6)

    y2 = np.array(())
    for i in range(5):
        xi1 = np.linspace(-0.5 * np.pi, 0.5 * np.pi, 10)
        yi1 = np.sin(xi1)
        yi1 = custom_normalize(yi1, 0, 1)
        
        yi2 = np.ones(20) 
        
        xi3 = np.linspace(0.5 * np.pi, 1.5 * np.pi, 10)
        yi3 = np.sin(xi3)
        yi3 = custom_normalize(yi3, 0, 1)
        
        yi4 = np.zeros(20) 
        
        yii = np.concatenate((yi1, yi2, yi3, yi4))
        yii = custom_normalize(yii, 0.6, 1)
        
        yii = yii + np.random.normal(0, 0.01, len(yii))
        y2 = np.concatenate((y2, yii))

    x3 = np.linspace(1, 2, 20)
    y3 = 1 / x3
    y3 = y3 + np.random.normal(0, 0.03, len(x1))
    y3 = custom_normalize(y3, 0, 0.6)

    y = np.concatenate((y1, y2, y3))
    waves = 3600 * y
    waves = list(waves)
    return waves

def mountain_curve():
    MAXIMUM = 3600
    def get_next(waves, current):
        flat = np.random.choice([True] * 9 + [False] * 2)
        if flat:
            delta = 0
        else:
            delta = 10 * np.random.normal(0, 20+10/(1+abs(current - MAXIMUM/2)), 1)[0]
        next_point = np.minimum(np.maximum(current + delta, 0), MAXIMUM)
        steps = np.random.randint(1, 100) 
        for step in range(steps):
            point = current + step/steps * (next_point - current)
            waves.append(point)
    waves = [MAXIMUM / 2]
    for i in range(10000):
        get_next(waves, waves[-1])
    return waves

MEMORY_BLOCK_MB = 10
tensor_list = []

def get_gpu_memory_usage():
    try:
        result = subprocess.run(['nvidia-smi', '--query-gpu=memory.used', '--format=csv,noheader,nounits'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
        
        if result.returncode == 0:
            memory_used = result.stdout.strip()
            return int(memory_used)
        else:
            error_message = result.stderr.strip()
            log("Error:", error_message)
            return None
    except Exception as e:
        log("An error occurred:", e)
        return None

def allocate_memory():
    tensor_size = int(MEMORY_BLOCK_MB * 1024 * 1024 / 4)  # Convert MB to elements (assuming float32 tensor)
    tensor = torch.rand(tensor_size).cuda()
    return tensor

def operation(diff):
    if diff > 0:
        tensor = allocate_memory()
        tensor_list.append(tensor)
    else:
        if len(tensor_list) == 0:
            return
        tensor = tensor_list.pop()
        del tensor
        torch.cuda.empty_cache()

def reach_to(target_memory, sleep):
    diff = target_memory - get_gpu_memory_usage()
    log("diff {}".format(diff))
    block_num = abs(int(diff / MEMORY_BLOCK_MB))
    for i in range(block_num):
        operation(diff)
    os.system("nvidia-smi")
    log("sleep for {}".format(sleep))
    time.sleep(sleep)

def memory_management_loop(waves, sleep):
    for wave in waves:
        log("next wave {}".format(wave))
        reach_to(wave, sleep)

def main():
    # Training settings
    parser = argparse.ArgumentParser(description='PyTorch MEMORY Example')
    parser.add_argument('--waves', type=str, default="0", metavar='N',
                        help='waves (default: sine)')
    parser.add_argument('--sleep', type=int, default=30, metavar='N',
                        help='sleep (default: 30)')
    args = parser.parse_args()
    waves = [int(num.strip()) for num in args.waves.split(",") if num] 
    if len(waves) <= 1:
        if waves[0] == 0:
            waves = sine_curve()
        elif waves[0] == 1:
            waves = mountain_curve()
    log("waves {}".format(waves))
    log("sleep {}".format(args.sleep))

    memory_management_loop(waves, args.sleep)
    
if __name__ == "__main__":
    main()

